@@ -14,21 +14,22 @@
 
 struct _NautilusAppChooser
 {
-    GtkDialog parent_instance;
+    AdwDialog parent_instance;
 
     gchar *content_type;
     gchar *file_name;
     gboolean single_content_type;
 
+    GtkWidget *ok_button;
     GtkWidget *content_box;
     GtkWidget *label_description;
+    GtkWidget *set_default_list_box;
     GtkWidget *set_default_row;
-    GtkWidget *set_default_box;
 
     GtkWidget *app_chooser_widget;
 };
 
-G_DEFINE_TYPE (NautilusAppChooser, nautilus_app_chooser, GTK_TYPE_DIALOG)
+G_DEFINE_TYPE (NautilusAppChooser, nautilus_app_chooser, ADW_TYPE_DIALOG)
 
 enum
 {
@@ -39,6 +40,14 @@ enum
     LAST_PROP
 };
 
+enum
+{
+    SIGNAL_APP_SELECTED,
+    SIGNAL_LAST
+};
+
+static guint signals[SIGNAL_LAST] = { 0, };
+
 static void
 open_cb (NautilusAppChooser *self)
 {
@@ -46,6 +55,8 @@ open_cb (NautilusAppChooser *self)
     g_autoptr (GAppInfo) info = NULL;
     g_autoptr (GError) error = NULL;
 
+    info = nautilus_app_chooser_get_app_info (self);
+
     if (!self->single_content_type)
     {
         /* Don't attempt to set an association with multiple content types */
@@ -60,7 +71,6 @@ open_cb (NautilusAppChooser *self)
 
     if (set_new_default)
     {
-        info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (self->app_chooser_widget));
         g_app_info_set_as_default_for_type (info, self->content_type,
                                             &error);
         g_signal_emit_by_name (nautilus_signaller_get_current (), "mime-data-changed");
@@ -68,24 +78,24 @@ open_cb (NautilusAppChooser *self)
 
     if (error != NULL)
     {
-        g_autofree gchar *message = NULL;
-        GtkWidget *message_dialog;
+        AdwAlertDialog *dialog;
 
-        message = g_strdup_printf (_("Error while setting “%s” as default app: %s"),
-                                   g_app_info_get_display_name (info), error->message);
-        message_dialog = adw_message_dialog_new (GTK_WINDOW (self),
-                                                 _("Could not set as default"),
-                                                 message);
-        adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (message_dialog), "close", _("OK"));
-        gtk_window_present (GTK_WINDOW (message_dialog));
+        dialog = ADW_ALERT_DIALOG (adw_alert_dialog_new (_("Could not set as default"), NULL));
+        adw_alert_dialog_format_body (dialog,
+                                      _("Error while setting “%s” as default app: %s"),
+                                      g_app_info_get_display_name (info), error->message);
+        adw_alert_dialog_add_response (dialog, "close", _("_OK"));
+
+        adw_dialog_present (ADW_DIALOG (dialog), GTK_WIDGET (self));
     }
+
+    g_signal_emit (self, signals[SIGNAL_APP_SELECTED], 0, info);
 }
 
 static void
 on_application_activated (NautilusAppChooser *self)
 {
     open_cb (self);
-    gtk_dialog_response (GTK_DIALOG (self), GTK_RESPONSE_OK);
 }
 
 static void
@@ -97,7 +107,7 @@ on_application_selected (GtkAppChooserWidget *widget,
     g_autoptr (GAppInfo) default_app = NULL;
     gboolean is_default;
 
-    gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, info != NULL);
+    gtk_widget_set_sensitive (self->ok_button, info != NULL);
 
     default_app = g_app_info_get_default_for_type (self->content_type, FALSE);
     is_default = default_app != NULL && g_app_info_equal (info, default_app);
@@ -106,6 +116,29 @@ on_application_selected (GtkAppChooserWidget *widget,
     gtk_widget_set_sensitive (GTK_WIDGET (self->set_default_row), !is_default);
 }
 
+static void
+focus_app_chooser_widget (NautilusAppChooser *self)
+{
+    /* This is a very hacky way to make focusing on the app chooser widget work.
+     * The widget is deprecated anyway and intended to be replaced by a new
+     * implementation, so we'll live with this rather than patching GTK.
+     */
+
+    GtkWidget *child = gtk_widget_get_first_child (self->app_chooser_widget);
+    g_return_if_fail (GTK_IS_OVERLAY (child));
+
+    child = gtk_widget_get_first_child (child);
+    g_return_if_fail (GTK_IS_SCROLLED_WINDOW (child));
+
+    child = gtk_widget_get_first_child (child);
+    g_return_if_fail (GTK_IS_TREE_VIEW (child));
+
+    gtk_widget_grab_focus (child);
+
+    /* Matching ref of timeout creation */
+    g_object_unref (self);
+}
+
 static void
 nautilus_app_chooser_set_property (GObject      *object,
                                    guint         param_id,
@@ -175,8 +208,14 @@ nautilus_app_chooser_constructed (GObject *object)
     gtk_app_chooser_widget_set_show_fallback (GTK_APP_CHOOSER_WIDGET (self->app_chooser_widget), TRUE);
     gtk_app_chooser_widget_set_show_other (GTK_APP_CHOOSER_WIDGET (self->app_chooser_widget), TRUE);
 
+    /* See comment in focus_app_chooser_widget(). Hold self reference to prevent segfaults. */
+    guint upper_dialog_creation_estimate = 100;
+    g_timeout_add_once (upper_dialog_creation_estimate,
+                        (GSourceOnceFunc) focus_app_chooser_widget,
+                        g_object_ref (self));
+
     /* initialize sensitivity */
-    info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (self->app_chooser_widget));
+    info = nautilus_app_chooser_get_app_info (self);
     if (info != NULL)
     {
         on_application_selected (GTK_APP_CHOOSER_WIDGET (self->app_chooser_widget),
@@ -208,8 +247,7 @@ nautilus_app_chooser_constructed (GObject *object)
         title = _("Open File");
     }
 
-    gtk_header_bar_set_title_widget (GTK_HEADER_BAR (gtk_dialog_get_header_bar (GTK_DIALOG (self))),
-                                     adw_window_title_new (title, NULL));
+    adw_dialog_set_title (ADW_DIALOG (self), title);
 
     if (self->single_content_type && !content_type_is_folder (self))
     {
@@ -218,7 +256,7 @@ nautilus_app_chooser_constructed (GObject *object)
     }
     else
     {
-        gtk_widget_set_visible (self->set_default_box, FALSE);
+        gtk_widget_set_visible (self->set_default_list_box, FALSE);
     }
 }
 
@@ -256,10 +294,19 @@ nautilus_app_chooser_class_init (NautilusAppChooserClass *klass)
 
     gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/nautilus/ui/nautilus-app-chooser.ui");
 
+    signals[SIGNAL_APP_SELECTED] = g_signal_new ("app-selected",
+                                                 NAUTILUS_TYPE_APP_CHOOSER,
+                                                 G_SIGNAL_RUN_LAST,
+                                                 0, NULL, NULL, NULL,
+                                                 G_TYPE_NONE,
+                                                 0,
+                                                 NULL);
+
+    gtk_widget_class_bind_template_child (widget_class, NautilusAppChooser, ok_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusAppChooser, content_box);
     gtk_widget_class_bind_template_child (widget_class, NautilusAppChooser, label_description);
+    gtk_widget_class_bind_template_child (widget_class, NautilusAppChooser, set_default_list_box);
     gtk_widget_class_bind_template_child (widget_class, NautilusAppChooser, set_default_row);
-    gtk_widget_class_bind_template_child (widget_class, NautilusAppChooser, set_default_box);
 
     gtk_widget_class_bind_template_callback (widget_class, open_cb);
 
@@ -283,8 +330,7 @@ nautilus_app_chooser_class_init (NautilusAppChooserClass *klass)
 }
 
 NautilusAppChooser *
-nautilus_app_chooser_new (GList     *files,
-                          GtkWindow *parent_window)
+nautilus_app_chooser_new (GList *files)
 {
     gboolean single_content_type = TRUE;
     const char *content_type = nautilus_file_get_mime_type (files->data);
@@ -300,9 +346,7 @@ nautilus_app_chooser_new (GList     *files,
     }
 
     return NAUTILUS_APP_CHOOSER (g_object_new (NAUTILUS_TYPE_APP_CHOOSER,
-                                               "transient-for", parent_window,
                                                "content-type", content_type,
-                                               "use-header-bar", TRUE,
                                                "file-name", file_name,
                                                "single-content-type", single_content_type,
                                                NULL));
