@@ -36,6 +36,7 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
+#include <libportal/portal.h>
 #include <nautilus-extension.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -51,6 +52,7 @@
 #include "nautilus-file-operations.h"
 #include "nautilus-file-undo-manager.h"
 #include "nautilus-file-utilities.h"
+#include "nautilus-files-view.h"
 #include "nautilus-freedesktop-dbus.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-icon-info.h"
@@ -63,15 +65,16 @@
 #include "nautilus-shell-search-provider.h"
 #include "nautilus-signaller.h"
 #include "nautilus-tag-manager.h"
-#include "nautilus-tracker-utilities.h"
+#include "nautilus-localsearch-utilities.h"
 #include "nautilus-trash-monitor.h"
 #include "nautilus-ui-utilities.h"
-#include "nautilus-view.h"
 #include "nautilus-window-slot.h"
 #include "nautilus-window.h"
 
-typedef struct
+struct _NautilusApplication
 {
+    AdwApplication parent_instance;
+
     NautilusProgressPersistenceHandler *progress_handler;
     NautilusDBusManager *dbus_manager;
     NautilusFreedesktopDBus *fdb_manager;
@@ -92,9 +95,9 @@ typedef struct
     GUnixMountMonitor *mount_monitor;
 
     NautilusDBusLauncher *dbus_launcher;
-} NautilusApplicationPrivate;
+};
 
-G_DEFINE_TYPE_WITH_PRIVATE (NautilusApplication, nautilus_application, ADW_TYPE_APPLICATION);
+G_DEFINE_FINAL_TYPE (NautilusApplication, nautilus_application, ADW_TYPE_APPLICATION)
 
 void
 nautilus_application_set_accelerator (GApplication *app,
@@ -121,26 +124,18 @@ nautilus_application_set_accelerators (GApplication  *app,
 GList *
 nautilus_application_get_windows (NautilusApplication *self)
 {
-    NautilusApplicationPrivate *priv;
-
-    priv = nautilus_application_get_instance_private (self);
-
-    return priv->windows;
+    return self->windows;
 }
 
 NautilusBookmarkList *
 nautilus_application_get_bookmarks (NautilusApplication *self)
 {
-    NautilusApplicationPrivate *priv;
-
-    priv = nautilus_application_get_instance_private (self);
-
-    if (!priv->bookmark_list)
+    if (!self->bookmark_list)
     {
-        priv->bookmark_list = nautilus_bookmark_list_new ();
+        self->bookmark_list = nautilus_bookmark_list_new ();
     }
 
-    return priv->bookmark_list;
+    return self->bookmark_list;
 }
 
 static gboolean
@@ -242,7 +237,8 @@ menu_provider_init_callback (void)
 }
 
 NautilusWindow *
-nautilus_application_create_window (NautilusApplication *self)
+nautilus_application_create_window (NautilusApplication *self,
+                                    const char          *startup_id)
 {
     NautilusWindow *window;
     gboolean maximized;
@@ -253,6 +249,10 @@ nautilus_application_create_window (NautilusApplication *self)
     g_return_val_if_fail (NAUTILUS_IS_APPLICATION (self), NULL);
 
     window = nautilus_window_new ();
+    if (startup_id)
+    {
+        gtk_window_set_startup_id (GTK_WINDOW (window), startup_id);
+    }
 
     maximized = g_settings_get_boolean
                     (nautilus_window_state, NAUTILUS_WINDOW_STATE_MAXIMIZED);
@@ -287,7 +287,6 @@ static NautilusWindowSlot *
 get_window_slot_for_location (NautilusApplication *self,
                               GFile               *location)
 {
-    NautilusApplicationPrivate *priv = nautilus_application_get_instance_private (self);
     g_autofree char *uri = g_file_get_uri (location);
     g_autoptr (NautilusFile) file = nautilus_file_get_existing (location);
     g_autoptr (GFile) searched_location = NULL;
@@ -303,7 +302,7 @@ get_window_slot_for_location (NautilusApplication *self,
         searched_location = g_object_ref (location);
     }
 
-    for (GList *l = priv->windows; l != NULL; l = l->next)
+    for (GList *l = self->windows; l != NULL; l = l->next)
     {
         NautilusWindow *window = l->data;
 
@@ -333,28 +332,10 @@ void
 nautilus_application_open_location_full (NautilusApplication *self,
                                          GFile               *location,
                                          NautilusOpenFlags    flags,
-                                         GList               *selection,
+                                         NautilusFileList    *selection,
                                          NautilusWindow      *target_window,
                                          NautilusWindowSlot  *target_slot,
                                          const char          *startup_id)
-{
-    NAUTILUS_APPLICATION_CLASS (G_OBJECT_GET_CLASS (self))->open_location_full (self,
-                                                                                location,
-                                                                                flags,
-                                                                                selection,
-                                                                                target_window,
-                                                                                target_slot,
-                                                                                startup_id);
-}
-
-static void
-real_open_location_full (NautilusApplication *self,
-                         GFile               *location,
-                         NautilusOpenFlags    flags,
-                         GList               *selection,
-                         NautilusWindow      *target_window,
-                         NautilusWindowSlot  *target_slot,
-                         const char          *startup_id)
 {
     NautilusWindowSlot *active_slot = NULL;
     NautilusWindow *active_window;
@@ -367,7 +348,7 @@ real_open_location_full (NautilusApplication *self,
     /* FIXME: We are having problems on getting the current focused window with
      * gtk_application_get_active_window, see https://bugzilla.gnome.org/show_bug.cgi?id=756499
      * so what we do is never rely on this on the callers, but would be cool to
-     * make it work withouth explicitly setting the active window on the callers. */
+     * make it work without explicitly setting the active window on the callers. */
     active_window = NAUTILUS_WINDOW (gtk_application_get_active_window (GTK_APPLICATION (self)));
     /* There is no active window if the application is run with
      * --gapplication-service
@@ -439,7 +420,7 @@ real_open_location_full (NautilusApplication *self,
                   gtk_root_get_display (GTK_ROOT (active_window)) :
                   gdk_display_get_default ();
 
-        target_window = nautilus_application_create_window (self);
+        target_window = nautilus_application_create_window (self, startup_id);
         /* Whatever the caller says, the slot won't be the same */
         gtk_window_set_display (GTK_WINDOW (target_window), display);
         target_slot = NULL;
@@ -450,14 +431,14 @@ real_open_location_full (NautilusApplication *self,
     /* Application is the one that manages windows, so this flag shouldn't use
      * it anymore by any client */
     flags &= ~NAUTILUS_OPEN_FLAG_NEW_WINDOW;
-    nautilus_window_open_location_full (target_window, location, flags, selection, target_slot, startup_id);
+    nautilus_window_open_location_full (target_window, location, flags, selection, target_slot);
 }
 
 static NautilusWindow *
 open_window (NautilusApplication *self,
              GFile               *location)
 {
-    NautilusWindow *window = nautilus_application_create_window (self);
+    NautilusWindow *window = nautilus_application_create_window (self, NULL);
 
     if (location != NULL)
     {
@@ -483,7 +464,7 @@ nautilus_application_open_location (NautilusApplication *self,
 {
     NautilusWindow *window;
     NautilusWindowSlot *slot;
-    GList *sel_list = NULL;
+    g_autolist (NautilusFile) sel_list = NULL;
     g_autofree char *location_uri = g_file_get_uri (location);
 
     if (location_uri[0] == '\0')
@@ -500,7 +481,7 @@ nautilus_application_open_location (NautilusApplication *self,
 
     if (!slot)
     {
-        window = nautilus_application_create_window (self);
+        window = nautilus_application_create_window (self, startup_id);
     }
     else
     {
@@ -508,11 +489,6 @@ nautilus_application_open_location (NautilusApplication *self,
     }
 
     nautilus_application_open_location_full (self, location, 0, sel_list, window, slot, startup_id);
-
-    if (sel_list != NULL)
-    {
-        nautilus_file_list_free (sel_list);
-    }
 }
 
 /* Note: when launched from command line we do not reach this method
@@ -560,28 +536,24 @@ nautilus_application_open (GApplication  *app,
 static void
 nautilus_application_finalize (GObject *object)
 {
-    NautilusApplication *self;
-    NautilusApplicationPrivate *priv;
+    NautilusApplication *self = NAUTILUS_APPLICATION (object);
 
-    self = NAUTILUS_APPLICATION (object);
-    priv = nautilus_application_get_instance_private (self);
+    g_clear_object (&self->progress_handler);
+    g_clear_object (&self->bookmark_list);
 
-    g_clear_object (&priv->progress_handler);
-    g_clear_object (&priv->bookmark_list);
+    g_list_free (self->windows);
 
-    g_list_free (priv->windows);
-
-    g_hash_table_destroy (priv->notifications);
+    g_hash_table_destroy (self->notifications);
 
     nautilus_module_teardown ();
 
-    g_clear_object (&priv->undo_manager);
+    g_clear_object (&self->undo_manager);
 
-    g_clear_object (&priv->tag_manager);
+    g_clear_object (&self->tag_manager);
 
-    g_clear_object (&priv->mount_monitor);
+    g_clear_object (&self->mount_monitor);
 
-    g_clear_object (&priv->dbus_launcher);
+    g_clear_object (&self->dbus_launcher);
 
     nautilus_trash_monitor_clear ();
 
@@ -589,17 +561,14 @@ nautilus_application_finalize (GObject *object)
 }
 
 static gboolean
-do_cmdline_sanity_checks (NautilusApplication *self,
-                          GVariantDict        *options)
+do_cmdline_sanity_checks (GVariantDict *options)
 {
-    gboolean retval = FALSE;
-
     if (g_variant_dict_contains (options, "quit") &&
         g_variant_dict_contains (options, G_OPTION_REMAINING))
     {
         g_printerr ("%s\n",
                     _("--quit cannot be used with URIs."));
-        goto out;
+        return FALSE;
     }
 
 
@@ -608,13 +577,10 @@ do_cmdline_sanity_checks (NautilusApplication *self,
     {
         g_printerr ("%s\n",
                     _("--select must be used with at least an URI."));
-        goto out;
+        return FALSE;
     }
 
-    retval = TRUE;
-
-out:
-    return retval;
+    return TRUE;
 }
 
 static void
@@ -663,18 +629,14 @@ action_clone_window (GSimpleAction *action,
                      GVariant      *parameter,
                      gpointer       user_data)
 {
-    NautilusWindowSlot *active_slot = NULL;
-    NautilusWindow *active_window = NULL;
     GtkApplication *application = user_data;
+    NautilusWindow *active_window = NAUTILUS_WINDOW (gtk_application_get_active_window (application));
+    NautilusWindowSlot *active_slot = nautilus_window_get_active_slot (active_window);
+    NautilusFilesView *current_view = nautilus_window_slot_get_current_view (active_slot);
     g_autoptr (GFile) location = NULL;
-    NautilusView *current_view;
-
-    active_window = NAUTILUS_WINDOW (gtk_application_get_active_window (application));
-    active_slot = nautilus_window_get_active_slot (active_window);
-    current_view = nautilus_window_slot_get_current_view (active_slot);
 
     if (current_view != NULL &&
-        nautilus_view_is_searching (current_view))
+        nautilus_files_view_is_searching (current_view))
     {
         location = g_file_new_for_path (g_get_home_dir ());
     }
@@ -920,37 +882,29 @@ nautilus_application_command_line (GApplication            *application,
                                    GApplicationCommandLine *command_line)
 {
     NautilusApplication *self = NAUTILUS_APPLICATION (application);
-    gint retval = -1;
-    GVariantDict *options;
-
-    options = g_application_command_line_get_options_dict (command_line);
+    GVariantDict *options = g_application_command_line_get_options_dict (command_line);
 
     if (g_variant_dict_contains (options, "version"))
     {
         g_application_command_line_print (command_line,
                                           "GNOME nautilus " PACKAGE_VERSION "\n");
-        retval = EXIT_SUCCESS;
-        goto out;
+        return EXIT_SUCCESS;
     }
-
-    if (!do_cmdline_sanity_checks (self, options))
+    else if (!do_cmdline_sanity_checks (options))
     {
-        retval = EXIT_FAILURE;
-        goto out;
+        return EXIT_FAILURE;
     }
-
-    if (g_variant_dict_contains (options, "quit"))
+    else if (g_variant_dict_contains (options, "quit"))
     {
         g_debug ("Killing app, as requested");
         g_action_group_activate_action (G_ACTION_GROUP (application),
                                         "kill", NULL);
-        goto out;
+        return -1;
+    }
+    else
+    {
+        return nautilus_application_handle_file_args (self, options);
     }
-
-    retval = nautilus_application_handle_file_args (self, options);
-
-out:
-    return retval;
 }
 
 static void
@@ -998,26 +952,23 @@ nautilus_application_init (NautilusApplication *self)
 
         { NULL }
     };
-    NautilusApplicationPrivate *priv;
 
-    priv = nautilus_application_get_instance_private (self);
-
-    priv->notifications = g_hash_table_new_full (g_str_hash,
+    self->notifications = g_hash_table_new_full (g_str_hash,
                                                  g_str_equal,
                                                  g_free,
                                                  NULL);
 
-    priv->undo_manager = nautilus_file_undo_manager_new ();
-    priv->tag_manager = nautilus_tag_manager_new ();
+    self->undo_manager = nautilus_file_undo_manager_new ();
+    self->tag_manager = nautilus_tag_manager_new ();
 
     /* Retain a mount monitor so GIO's caching works. This helps to speed
      * up various filesystem queries, e.g. those determining whether to
      * recurse during searches. */
-    priv->mount_monitor = g_unix_mount_monitor_get ();
+    self->mount_monitor = g_unix_mount_monitor_get ();
 
-    priv->dbus_launcher = nautilus_dbus_launcher_new ();
+    self->dbus_launcher = nautilus_dbus_launcher_new ();
 
-    nautilus_tracker_setup_miner_fs_connection ();
+    nautilus_localsearch_setup_miner_fs_connection ();
 
     g_application_add_main_option_entries (G_APPLICATION (self), options);
 
@@ -1042,11 +993,7 @@ nautilus_application_send_notification (NautilusApplication *self,
                                         const gchar         *notification_id,
                                         GNotification       *notification)
 {
-    NautilusApplicationPrivate *priv;
-
-    priv = nautilus_application_get_instance_private (self);
-
-    g_hash_table_add (priv->notifications, g_strdup (notification_id));
+    g_hash_table_add (self->notifications, g_strdup (notification_id));
     g_application_send_notification (G_APPLICATION (self), notification_id, notification);
 }
 
@@ -1054,15 +1001,12 @@ void
 nautilus_application_withdraw_notification (NautilusApplication *self,
                                             const gchar         *notification_id)
 {
-    NautilusApplicationPrivate *priv;
-
-    priv = nautilus_application_get_instance_private (self);
-    if (!g_hash_table_contains (priv->notifications, notification_id))
+    if (!g_hash_table_contains (self->notifications, notification_id))
     {
         return;
     }
 
-    g_hash_table_remove (priv->notifications, notification_id);
+    g_hash_table_remove (self->notifications, notification_id);
     g_application_withdraw_notification (G_APPLICATION (self), notification_id);
 }
 
@@ -1071,13 +1015,11 @@ on_application_shutdown (GApplication *application,
                          gpointer      user_data)
 {
     NautilusApplication *self = NAUTILUS_APPLICATION (application);
-    NautilusApplicationPrivate *priv;
     GList *notification_ids;
     GList *l;
     gchar *notification_id;
 
-    priv = nautilus_application_get_instance_private (self);
-    notification_ids = g_hash_table_get_keys (priv->notifications);
+    notification_ids = g_hash_table_get_keys (self->notifications);
     for (l = notification_ids; l != NULL; l = l->next)
     {
         notification_id = l->data;
@@ -1167,7 +1109,6 @@ static void
 nautilus_application_startup (GApplication *app)
 {
     NautilusApplication *self = NAUTILUS_APPLICATION (app);
-    NautilusApplicationPrivate *priv = nautilus_application_get_instance_private (self);
 
     g_application_set_resource_base_path (G_APPLICATION (self), "/org/gnome/nautilus");
 
@@ -1213,7 +1154,7 @@ nautilus_application_startup (GApplication *app)
     menu_provider_init_callback ();
 
     /* Initialize the UI handler singleton for file operations */
-    priv->progress_handler = nautilus_progress_persistence_handler_new (G_OBJECT (self));
+    self->progress_handler = nautilus_progress_persistence_handler_new (G_OBJECT (self));
 
     /* Check the user's .nautilus directories and post warnings
      * if there are problems.
@@ -1242,32 +1183,30 @@ nautilus_application_dbus_register (GApplication     *app,
                                     GError          **error)
 {
     NautilusApplication *self = NAUTILUS_APPLICATION (app);
-    NautilusApplicationPrivate *priv;
 
-    priv = nautilus_application_get_instance_private (self);
-    priv->dbus_manager = nautilus_dbus_manager_new ();
-    if (!nautilus_dbus_manager_register (priv->dbus_manager, connection, error))
+    self->dbus_manager = nautilus_dbus_manager_new ();
+    if (!nautilus_dbus_manager_register (self->dbus_manager, connection, error))
     {
         return FALSE;
     }
 
-    priv->fdb_manager = nautilus_freedesktop_dbus_new ();
-    if (!nautilus_freedesktop_dbus_register (priv->fdb_manager, connection, error))
+    self->fdb_manager = nautilus_freedesktop_dbus_new ();
+    if (!nautilus_freedesktop_dbus_register (self->fdb_manager, connection, error))
     {
         return FALSE;
     }
 
     if (g_strcmp0 (g_getenv ("RUNNING_TESTS"), "TRUE") != 0)
     {
-        priv->portal_implementation = nautilus_portal_new ();
-        if (!nautilus_portal_register (priv->portal_implementation, connection, error))
+        self->portal_implementation = nautilus_portal_new ();
+        if (!nautilus_portal_register (self->portal_implementation, connection, error))
         {
             return FALSE;
         }
     }
 
-    priv->search_provider = nautilus_shell_search_provider_new ();
-    if (!nautilus_shell_search_provider_register (priv->search_provider, connection, error))
+    self->search_provider = nautilus_shell_search_provider_new ();
+    if (!nautilus_shell_search_provider_register (self->search_provider, connection, error))
     {
         return FALSE;
     }
@@ -1283,31 +1222,29 @@ nautilus_application_dbus_unregister (GApplication    *app,
                                       const gchar     *object_path)
 {
     NautilusApplication *self = NAUTILUS_APPLICATION (app);
-    NautilusApplicationPrivate *priv;
 
-    priv = nautilus_application_get_instance_private (self);
-    if (priv->dbus_manager)
+    if (self->dbus_manager)
     {
-        nautilus_dbus_manager_unregister (priv->dbus_manager);
-        g_clear_object (&priv->dbus_manager);
+        nautilus_dbus_manager_unregister (self->dbus_manager);
+        g_clear_object (&self->dbus_manager);
     }
 
-    if (priv->fdb_manager)
+    if (self->fdb_manager)
     {
-        nautilus_freedesktop_dbus_unregister (priv->fdb_manager);
-        g_clear_object (&priv->fdb_manager);
+        nautilus_freedesktop_dbus_unregister (self->fdb_manager);
+        g_clear_object (&self->fdb_manager);
     }
 
-    if (priv->portal_implementation != NULL)
+    if (self->portal_implementation != NULL)
     {
-        nautilus_portal_unregister (priv->portal_implementation);
-        g_clear_object (&priv->portal_implementation);
+        nautilus_portal_unregister (self->portal_implementation);
+        g_clear_object (&self->portal_implementation);
     }
 
-    if (priv->search_provider)
+    if (self->search_provider)
     {
-        nautilus_shell_search_provider_unregister (priv->search_provider);
-        g_clear_object (&priv->search_provider);
+        nautilus_shell_search_provider_unregister (self->search_provider);
+        g_clear_object (&self->search_provider);
     }
 
     nautilus_previewer_teardown (connection);
@@ -1316,7 +1253,6 @@ nautilus_application_dbus_unregister (GApplication    *app,
 static void
 update_dbus_opened_locations (NautilusApplication *self)
 {
-    NautilusApplicationPrivate *priv;
     gint i;
     GList *l, *sl;
     GList *locations = NULL;
@@ -1331,14 +1267,13 @@ update_dbus_opened_locations (NautilusApplication *self)
 
     g_return_if_fail (NAUTILUS_IS_APPLICATION (self));
 
-    priv = nautilus_application_get_instance_private (self);
     dbus_object_path = g_application_get_dbus_object_path (G_APPLICATION (self));
 
     g_return_if_fail (dbus_object_path);
 
     g_variant_builder_init (&windows_to_locations_builder, G_VARIANT_TYPE ("a{sas}"));
 
-    for (l = priv->windows; l != NULL; l = l->next)
+    for (l = self->windows; l != NULL; l = l->next)
     {
         guint32 id;
         g_autofree gchar *path = NULL;
@@ -1388,11 +1323,11 @@ update_dbus_opened_locations (NautilusApplication *self)
 
     locations_array[locations_size] = NULL;
 
-    nautilus_freedesktop_dbus_set_open_locations (priv->fdb_manager,
+    nautilus_freedesktop_dbus_set_open_locations (self->fdb_manager,
                                                   (const gchar **) locations_array);
 
     windows_to_locations = g_variant_ref_sink (g_variant_builder_end (&windows_to_locations_builder));
-    nautilus_freedesktop_dbus_set_open_windows_with_locations (priv->fdb_manager,
+    nautilus_freedesktop_dbus_set_open_windows_with_locations (self->fdb_manager,
                                                                windows_to_locations);
 
     g_free (locations_array);
@@ -1435,14 +1370,12 @@ nautilus_application_window_added (GtkApplication *app,
                                    GtkWindow      *window)
 {
     NautilusApplication *self = NAUTILUS_APPLICATION (app);
-    NautilusApplicationPrivate *priv;
 
-    priv = nautilus_application_get_instance_private (self);
     GTK_APPLICATION_CLASS (nautilus_application_parent_class)->window_added (app, window);
 
     if (NAUTILUS_IS_WINDOW (window))
     {
-        priv->windows = g_list_prepend (priv->windows, window);
+        self->windows = g_list_prepend (self->windows, window);
         g_signal_connect (window, "slot-added", G_CALLBACK (on_slot_added), app);
         g_signal_connect (window, "slot-removed", G_CALLBACK (on_slot_removed), app);
     }
@@ -1453,24 +1386,21 @@ nautilus_application_window_removed (GtkApplication *app,
                                      GtkWindow      *window)
 {
     NautilusApplication *self = NAUTILUS_APPLICATION (app);
-    NautilusApplicationPrivate *priv;
-
-    priv = nautilus_application_get_instance_private (self);
 
     GTK_APPLICATION_CLASS (nautilus_application_parent_class)->window_removed (app, window);
 
     if (NAUTILUS_IS_WINDOW (window))
     {
-        priv->windows = g_list_remove_all (priv->windows, window);
+        self->windows = g_list_remove_all (self->windows, window);
         g_signal_handlers_disconnect_by_func (window, on_slot_added, app);
         g_signal_handlers_disconnect_by_func (window, on_slot_removed, app);
     }
 
     /* if this was the last window, close the previewer */
-    if (priv->windows == NULL)
+    if (self->windows == NULL)
     {
         nautilus_previewer_call_close ();
-        nautilus_progress_persistence_handler_make_persistent (priv->progress_handler);
+        nautilus_progress_persistence_handler_make_persistent (self->progress_handler);
     }
 }
 
@@ -1510,8 +1440,6 @@ nautilus_application_class_init (NautilusApplicationClass *class)
     application_class->command_line = nautilus_application_command_line;
     application_class->handle_local_options = nautilus_application_handle_local_options;
 
-    class->open_location_full = real_open_location_full;
-
     gtkapp_class = GTK_APPLICATION_CLASS (class);
     gtkapp_class->window_added = nautilus_application_window_added;
     gtkapp_class->window_removed = nautilus_application_window_removed;
@@ -1531,11 +1459,7 @@ void
 nautilus_application_search (NautilusApplication *self,
                              NautilusQuery       *query)
 {
-    g_autoptr (GFile) location = NULL;
-    NautilusWindow *window;
-
-    location = nautilus_query_get_location (query);
-    window = open_window (self, location);
+    NautilusWindow *window = open_window (self, NULL);
     nautilus_window_search (window, query);
 }
 
@@ -1547,7 +1471,8 @@ nautilus_application_is_sandboxed (void)
     static gsize init = 0;
     if (g_once_init_enter (&init))
     {
-        ret = g_file_test ("/.flatpak-info", G_FILE_TEST_EXISTS);
+        ret = xdp_portal_running_under_sandbox ();
+
         g_once_init_leave (&init, 1);
     }
 
