@@ -6,12 +6,12 @@
 
 #pragma once
 
+#include "nautilus-types.h"
+
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <adwaita.h>
 
-#include "nautilus-directory.h"
-
 typedef void (*CompressCallback) (const char *new_name,
                                   const char *passphrase,
                                   gpointer    user_data);
