@@ -24,12 +24,12 @@
 
 #pragma once
 
+#include "nautilus-types.h"
+
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <gnome-autoar/gnome-autoar.h>
 
-#include "nautilus-file-operations-dbus-data.h"
-
 typedef enum
 {
     NAUTILUS_FILE_UNDO_OP_INVALID,
