@@ -101,8 +101,7 @@ nautilus_icon_info_class_init (NautilusIconInfoClass *icon_info_class)
 }
 
 NautilusIconInfo *
-nautilus_icon_info_new_for_paintable (GdkPaintable *paintable,
-                                      gint          scale)
+nautilus_icon_info_new_for_paintable (GdkPaintable *paintable)
 {
     NautilusIconInfo *icon;
 
@@ -117,14 +116,13 @@ nautilus_icon_info_new_for_paintable (GdkPaintable *paintable,
 }
 
 static NautilusIconInfo *
-nautilus_icon_info_new_for_icon_paintable (GtkIconPaintable *icon_paintable,
-                                           gint              scale)
+nautilus_icon_info_new_for_icon_paintable (GtkIconPaintable *icon_paintable)
 {
     NautilusIconInfo *icon;
     g_autoptr (GFile) file = NULL;
     char *basename, *p;
 
-    icon = nautilus_icon_info_new_for_paintable (GDK_PAINTABLE (icon_paintable), scale);
+    icon = nautilus_icon_info_new_for_paintable (GDK_PAINTABLE (icon_paintable));
 
     file = gtk_icon_paintable_get_file (icon_paintable);
     if (file != NULL)
@@ -383,7 +381,7 @@ nautilus_icon_info_lookup (GIcon *icon,
 
         lookup_key.icon = icon;
         lookup_key.scale = scale;
-        lookup_key.size = size;
+        lookup_key.size = size * scale;
 
         icon_info = g_hash_table_lookup (loadable_icon_cache, &lookup_key);
         if (icon_info)
@@ -417,7 +415,7 @@ nautilus_icon_info_lookup (GIcon *icon,
             paintable = gtk_snapshot_to_paintable (snapshot, NULL);
         }
 
-        icon_info = nautilus_icon_info_new_for_paintable (paintable, scale);
+        icon_info = nautilus_icon_info_new_for_paintable (paintable);
 
         key = loadable_icon_key_new (icon, scale, size);
         g_hash_table_insert (loadable_icon_cache, key, icon_info);
@@ -428,7 +426,7 @@ nautilus_icon_info_lookup (GIcon *icon,
     GtkIconTheme *theme = gtk_icon_theme_get_for_display (gdk_display_get_default ());
     if (!gtk_icon_theme_has_gicon (theme, icon))
     {
-        return nautilus_icon_info_new_for_paintable (NULL, scale);
+        return nautilus_icon_info_new_for_paintable (NULL);
     }
 
     icon_paintable = lookup_themed_icon (theme, icon, size, scale);
@@ -457,7 +455,7 @@ nautilus_icon_info_lookup (GIcon *icon,
         icon_info = g_hash_table_lookup (themed_icon_cache, &lookup_key);
         if (!icon_info)
         {
-            icon_info = nautilus_icon_info_new_for_icon_paintable (icon_paintable, scale);
+            icon_info = nautilus_icon_info_new_for_icon_paintable (icon_paintable);
 
             key = themed_icon_key_new (icon_name, scale, size);
             g_hash_table_insert (themed_icon_cache, key, icon_info);
@@ -467,7 +465,7 @@ nautilus_icon_info_lookup (GIcon *icon,
     }
     else
     {
-        return nautilus_icon_info_new_for_icon_paintable (icon_paintable, scale);
+        return nautilus_icon_info_new_for_icon_paintable (icon_paintable);
     }
 }
 
